@@ -4,30 +4,33 @@ use strict;
 
 use Apache::Session;
 use Apache::Session::Browseable::Store::Redis;
-use Apache::Session::Generate::MD5;
+use Apache::Session::Generate::SHA256;
 use Apache::Session::Lock::Null;
-use Apache::Session::Serialize::Base64;
+use Apache::Session::Serialize::JSON;
+use Apache::Session::Browseable::_common;
 
-our $VERSION = '0.9';
+our $VERSION = '1.3.2';
 our @ISA     = qw(Apache::Session);
 
+our $redis = $Apache::Session::Browseable::Store::Redis::redis;
+
 sub populate {
     my $self = shift;
 
     $self->{object_store} = new Apache::Session::Browseable::Store::Redis $self;
     $self->{lock_manager} = new Apache::Session::Lock::Null $self;
-    $self->{generate}     = \&Apache::Session::Generate::MD5::generate;
-    $self->{validate}     = \&Apache::Session::Generate::MD5::validate;
-    $self->{serialize}    = \&Apache::Session::Serialize::Base64::serialize;
-    $self->{unserialize}  = \&Apache::Session::Serialize::Base64::unserialize;
+    $self->{generate}     = \&Apache::Session::Generate::SHA256::generate;
+    $self->{validate}     = \&Apache::Session::Generate::SHA256::validate;
+    $self->{serialize}    = \&Apache::Session::Serialize::JSON::serialize;
+    $self->{unserialize}  = \&Apache::Session::Serialize::JSON::unserialize;
 
     return $self;
 }
 
 sub unserialize {
     my $session = shift;
-    my $tmp = { serialized => $session };
-    Apache::Session::Serialize::Base64::unserialize($tmp);
+    my $tmp     = { serialized => $session };
+    Apache::Session::Serialize::JSON::unserialize($tmp);
     return $tmp->{data};
 }
 
@@ -38,8 +41,9 @@ sub searchOn {
     my $index =
       ref( $args->{Index} ) ? $args->{Index} : [ split /\s+/, $args->{Index} ];
     if ( grep { $_ eq $selectField } @$index ) {
-        my $redisObj = Redis->new(%$args);
-        my @keys     = $redisObj->smembers("${selectField}_$value");
+
+        my $redisObj = $class->_getRedis($args);
+        my @keys = $redisObj->smembers("${selectField}_$value");
         foreach my $k (@keys) {
             next unless ($k);
             my $tmp = $redisObj->get($k);
@@ -59,7 +63,9 @@ sub searchOn {
             sub {
                 my $entry = shift;
                 my $id    = shift;
-                return undef unless ( defined $entry->{$selectField} and $entry->{$selectField} eq $value );
+                return undef
+                  unless ( defined $entry->{$selectField}
+                    and $entry->{$selectField} eq $value );
                 if (@fields) {
                     $res{$id}->{$_} = $entry->{$_} foreach (@fields);
                 }
@@ -79,9 +85,8 @@ sub get_key_from_all_sessions {
     my $data  = shift;
     my %res;
 
-    # TODO new Redis object
-    my $redisObj = Redis->new(%$args);
-    my @keys     = $redisObj->keys('*');
+    my $redisObj = $class->_getRedis($args);
+    my @keys = $redisObj->keys('*');
     foreach my $k (@keys) {
         next if ( !$k or $k =~ /_/ );
         my $v   = $redisObj->get($k);
@@ -101,6 +106,32 @@ sub get_key_from_all_sessions {
     return \%res;
 }
 
+sub _getRedis {
+    my $class = shift;
+    my $args  = shift;
+
+    # Manage undef encoding
+    $args->{encoding} = undef
+      if (  $args->{encoding}
+        and $args->{encoding} eq "undef" );
+
+    # If sentinels is not given as an array ref, try to parse
+    # a comma delimited list instead
+    if ( $args->{sentinels}
+        and ref $args->{sentinels} ne 'ARRAY' )
+    {
+        $args->{sentinels} =
+          [ split /[,\s]+/, $args->{sentinels} ];
+    }
+
+    my $redisObj = $redis->new( %{$args} );
+
+    # Manage database
+    $redisObj->select( $args->{database} )
+      if defined $args->{database};
+    return $redisObj;
+}
+
 1;
 __END__
 
@@ -162,6 +193,8 @@ Apache::Session::Redis
 Apache::Session::browseable provides some class methods to manipulate all
 sessions and add the capability to index some fields to make research faster.
 
+This module use either L<Redis::Fast> or L<Redis>.
+
 =head1 SEE ALSO
 
 L<Apache::Session>
@@ -172,10 +205,7 @@ Xavier Guimard, E<lt>x.guimard@free.frE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-=encoding utf8
-
-Copyright (C) 2009-2013 by Xavier Guimard
-              2013 by Clément Oudot
+Copyright (C) 2009-2018 by Xavier Guimard
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.10.1 or,
